package com.mortals.xhx.module.device.web;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.device.DeviceAlarmPdu;
import com.mortals.xhx.common.pdu.device.DeviceModuleUsePdu;
import com.mortals.xhx.common.pdu.device.DevicePdu;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.feign.device.IDeviceAlarmFeign;
import com.mortals.xhx.feign.device.IDeviceFeign;
import com.mortals.xhx.feign.device.IDeviceModuleUseFeign;
import com.mortals.xhx.module.device.model.DeviceEntity;
import com.mortals.xhx.module.device.service.DeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 设备
 *
 * @author zxfei
 * @date 2022-06-27
 */
@RestController
@RequestMapping("device")
@Slf4j
public class DeviceController extends BaseCRUDJsonBodyMappingController<DeviceService, DeviceEntity, Long> {

    @Autowired
    private IDeviceFeign deviceFeign;

    @Autowired
    private IDeviceAlarmFeign deviceAlarmFeign;

    @Autowired
    private IDeviceModuleUseFeign deviceModuleUseFeign;

    @Override
    @PostMapping({"list"})
    @UnAuth
    public Rest<Object> list(@RequestBody DeviceEntity query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询" + this.getModuleDesc();

        int code;
        try {
            PageInfo pageInfo = this.buildPageInfo(query);
            this.doListBefore(query, model, context);
            Result<DeviceEntity> result = this.getService().find(query, pageInfo, context);
            model.put("data", result.getList());
            int onlineTotal = 0;
            int offlineTotal = 0;
            int not_active = 0;
            int stop_using = 0;
            for(DeviceEntity deviceEntity:result.getList()){
                if(deviceEntity.getDeviceStatus()==0){
                    not_active++;
                }
                if(deviceEntity.getDeviceStatus()==1){
                    offlineTotal++;
                }
                if(deviceEntity.getDeviceStatus()==2){
                    onlineTotal++;
                }
                if(deviceEntity.getEnabled()==0){
                    stop_using++;
                }
            }
            model.put("online", onlineTotal);
            model.put("offline", offlineTotal);
            model.put("notActive", not_active);
            model.put("stopUsing", stop_using);
            model.put("pageInfo", result.getPageInfo());
            this.parsePageInfo(model, result.getPageInfo());
            code = this.doListAfter(query, model, context);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        this.init(model, context);
        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }


    @RequestMapping(value = {"info"},method = {RequestMethod.POST, RequestMethod.GET})
    @UnAuth
    public String info(Long id) {
        Map<String, Object> model = new HashMap();
        if (id == null) {
            return this.createFailJsonResp("请选择待查看" + this.getModuleDesc() + "信息");
        } else {
            JSONObject ret = new JSONObject();
            String busiDesc = "查看" + this.getModuleDesc();
            Context context = this.getContext();

            try {
                DeviceEntity deviceEntity = new DeviceEntity();
                Rest<DevicePdu> result = deviceFeign.info(id);
                if (result.getCode().equals(YesNoEnum.YES.getValue())) {
                    BeanUtils.copyProperties(result.getData(), deviceEntity, BeanUtil.getNullPropertyNames(result.getData()));
                    model.put("dict",result.getDict());
                    model.put("msg",result.getMsg());
                }
                if (deviceEntity == null) {
                    throw new Exception(busiDesc + "，不存在或已被删除");
                }

                ret.put("data", deviceEntity);
                this.recordSysLog(this.request, busiDesc + " 【成功】");
            } catch (Exception var8) {
                this.doException(this.request, busiDesc, model, var8);
                Object msg = model.get("message_info");
                return this.createFailJsonResp(msg == null ? "系统异常" : msg.toString());
            }

            this.init(model, context);
            ret.put("code", 1);
            ret.put("msg", model.remove("message_info"));
            ret.put("dict", model.remove("dict"));
            return ret.toJSONString();
        }
    }

    @PostMapping({"alarm/list"})
    @UnAuth
    public Rest<Object> alarmList(@RequestBody DeviceAlarmPdu query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询设备告警信息列表";

        int code=1;
        try {
            PageInfo pageInfo = new PageInfo();
            if (!ObjectUtils.isEmpty(query) && !ObjectUtils.isEmpty(query.getPage())) {
                pageInfo.setCurrPage(query.getPage());
            }

            if (!ObjectUtils.isEmpty(query) && !ObjectUtils.isEmpty(query.getSize())) {
                pageInfo.setPrePageResult(query.getSize());
            }
            Result<DeviceAlarmPdu> result = new Result();
            Rest<RespData<List<DeviceAlarmPdu>>> rest = deviceAlarmFeign.list(query);
            if (rest.getCode().equals(YesNoEnum.YES.getValue())) {
                List<DeviceAlarmPdu> collect = rest.getData().getData();
                result.setList(collect);
                pageInfo.setTotalResult(collect.size());
                result.setPageInfo(pageInfo);
                result.setDict(rest.getDict());
            }
            model.put("data", result.getList());
            model.put("pageInfo", result.getPageInfo());
            model.put("dict", result.getDict());
            this.parsePageInfo(model, result.getPageInfo());
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        this.init(model, context);
        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    @PostMapping({"module/use/list"})
    @UnAuth
    public Rest<Object> moduleUseList(@RequestBody DeviceModuleUsePdu query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询设备模块使用频率信息列表";

        int code=1;
        try {
            PageInfo pageInfo = new PageInfo();
            if (!ObjectUtils.isEmpty(query) && !ObjectUtils.isEmpty(query.getPage())) {
                pageInfo.setCurrPage(query.getPage());
            }

            if (!ObjectUtils.isEmpty(query) && !ObjectUtils.isEmpty(query.getSize())) {
                pageInfo.setPrePageResult(query.getSize());
            }
            Result<DeviceModuleUsePdu> result = new Result();
            Rest<RespData<List<DeviceModuleUsePdu>>> rest = deviceModuleUseFeign.list(query);
            if (rest.getCode().equals(YesNoEnum.YES.getValue())) {
                List<DeviceModuleUsePdu> collect = rest.getData().getData();
                result.setList(collect);
                pageInfo.setTotalResult(collect.size());
                result.setPageInfo(pageInfo);
                result.setDict(rest.getDict());
            }
            model.put("data", result.getList());
            model.put("pageInfo", result.getPageInfo());
            model.put("dict", result.getDict());
            this.parsePageInfo(model, result.getPageInfo());
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        this.init(model, context);
        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }
}