package com.mortals.xhx.module.sst.service.impl;
import com.mortals.framework.util.DataUtil;
import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.module.apps.model.AppsInfoEntity;
import com.mortals.xhx.module.apps.service.AppsInfoService;
import org.apache.commons.collections4.CollectionUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.sst.dao.SstAppsDao;
import com.mortals.xhx.module.sst.model.SstAppsEntity;
import com.mortals.xhx.module.sst.service.SstAppsService;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
* SstAppsService
* 应用编排 service实现
*
* @author zxfei
* @date 2022-12-21
*/
@Service("sstAppsService")
public class SstAppsServiceImpl extends AbstractCRUDServiceImpl<SstAppsDao, SstAppsEntity, Long> implements SstAppsService {

    @Autowired
    private AppsInfoService appsInfoService;

    @Override
    public Map<String, Object> getAppsBySiteId(Long siteId) throws AppException {
        if(siteId == null){
            throw new AppException("站点id不能为空");
        }
        AppsInfoEntity appsInfoQuery = new AppsInfoEntity();
        appsInfoQuery.setSiteId(siteId);
        List<AppsInfoEntity> allApps = appsInfoService.find(appsInfoQuery);
        SstAppsEntity query = new SstAppsEntity();
        query.setSiteId(siteId);
        List<SstAppsEntity> appsEntities = this.find(query);
        Map<String, Object> siteApp = new HashMap<>();
        siteApp.put("allApps", allApps);
        if(CollectionUtils.isNotEmpty(appsEntities)){
            List<SstAppsEntity> showApps = new ArrayList<>();
            List<SstAppsEntity> hotApps = new ArrayList<>();
            for(SstAppsEntity sstAppsEntity:appsEntities){
                if(sstAppsEntity.getShowBasic()==1){
                    showApps.add(sstAppsEntity);
                }
                if(sstAppsEntity.getShowHot()==1){
                    hotApps.add(sstAppsEntity);
                }
            }
            siteApp.put("showApps", showApps);
            siteApp.put("hotApps", hotApps);
        }else {
            siteApp.put("showApps", Collections.emptyList());
            siteApp.put("hotApps", Collections.emptyList());
        }
        return siteApp;
    }

    @Override
    public void settingSave(Long siteId, String showAppIds, String hotAppIds,Context context) throws AppException {
        Map<Long,SstAppsEntity> sstAppIds = new HashMap<>();

        if(StringUtils.isNotEmpty(showAppIds)){
            String appIds[] = showAppIds.split(",");
            int sort = 99;
            for(String id:appIds){
                Long appId = DataUtil.converStr2Long(id,0l);
                AppsInfoEntity appsInfoEntity = appsInfoService.get(appId);
                if(appsInfoEntity!=null) {
                    SstAppsEntity sstAppsEntity = new SstAppsEntity();
                    sstAppsEntity.setSiteId(siteId);
                    sstAppsEntity.setAppId(appId);
                    sstAppsEntity.setName(appsInfoEntity.getName());
                    sstAppsEntity.setDescribe(appsInfoEntity.getDescribe());
                    sstAppsEntity.setUrl(appsInfoEntity.getUrl());
                    sstAppsEntity.setShowBasic(1);
                    sstAppsEntity.setBasicSort(sort);
                    sstAppsEntity.setShowHot(0);
                    sstAppsEntity.setHotSort(0);
                    if(context!=null){
                        sstAppsEntity.setCreateUserId(context.getUser().getId());
                        sstAppsEntity.setCreateTime(new Date());
                    }
                    sstAppIds.put(appId, sstAppsEntity);
                    sort--;
                }
            }
        }
        if(StringUtils.isNotEmpty(hotAppIds)){
            String appIds[] = hotAppIds.split(",");
            int sort = 99;
            for(String id:appIds){
                Long appId = DataUtil.converStr2Long(id,0l);
                if(sstAppIds.containsKey(appId)){
                    sstAppIds.get(appId).setShowHot(1);
                    sstAppIds.get(appId).setHotSort(sort);
                    sort--;
                }else {
                    AppsInfoEntity appsInfoEntity = appsInfoService.get(appId);
                    if (appsInfoEntity != null) {
                        SstAppsEntity sstAppsEntity = new SstAppsEntity();
                        sstAppsEntity.setSiteId(siteId);
                        sstAppsEntity.setAppId(appId);
                        sstAppsEntity.setName(appsInfoEntity.getName());
                        sstAppsEntity.setDescribe(appsInfoEntity.getDescribe());
                        sstAppsEntity.setUrl(appsInfoEntity.getUrl());
                        sstAppsEntity.setShowBasic(0);
                        sstAppsEntity.setBasicSort(0);
                        sstAppsEntity.setShowHot(1);
                        sstAppsEntity.setHotSort(sort);
                        if (context != null) {
                            sstAppsEntity.setCreateUserId(context.getUser().getId());
                            sstAppsEntity.setCreateTime(new Date());
                        }
                        sstAppIds.put(appId, sstAppsEntity);
                        sort--;
                    }
                }
            }
        }
        Map<String,Object> condition = new HashMap<>();
        condition.put("siteId",siteId);
        this.dao.delete(condition);
        if(sstAppIds.size()>0){
            List<SstAppsEntity> list = sstAppIds.values().stream().collect(Collectors.toList());
            this.dao.insertBatch(list);
        }
    }

    @Override
    public Map<String, Object> getAppListBySite(Long siteId) {
        SstAppsEntity query = new SstAppsEntity();
        query.setSiteId(siteId);
        List<SstAppsEntity> appsEntities = this.find(query);

        AppsInfoEntity appsInfoQuery = new AppsInfoEntity();
        appsInfoQuery.setSiteId(siteId);
        List<AppsInfoEntity> allApps = appsInfoService.find(appsInfoQuery);
        Map<Long,AppsInfoEntity> appInfoMap = allApps.stream().collect(Collectors.toMap(AppsInfoEntity::getId, Function.identity()));
        Map<String, Object> siteApp = new HashMap<>();
        siteApp.put("allApps", allApps);
        if(CollectionUtils.isNotEmpty(appsEntities)){
            List<AppsInfoEntity> showApps = new ArrayList<>();
            List<AppsInfoEntity> hotApps = new ArrayList<>();
            for(SstAppsEntity sstAppsEntity:appsEntities){
                if(sstAppsEntity.getShowBasic()==1){
                    showApps.add(appInfoMap.get(sstAppsEntity.getAppId()));
                }
                if(sstAppsEntity.getShowHot()==1){
                    hotApps.add(appInfoMap.get(sstAppsEntity.getAppId()));
                }
            }
            siteApp.put("showApps", showApps);
            siteApp.put("hotApps", hotApps);
        }else {
            siteApp.put("showApps", Collections.emptyList());
            siteApp.put("hotApps", Collections.emptyList());
        }
        return siteApp;
    }
}