package com.mortals.xhx.module.sst.service.impl;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.device.DevicePdu;
import com.mortals.xhx.common.pdu.skinBase.SinkBasePdu;
import com.mortals.xhx.feign.device.IDeviceFeign;
import com.mortals.xhx.feign.skinBase.ISinkBaseFeign;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.sst.dao.SstSkinDao;
import com.mortals.xhx.module.sst.model.SstSkinEntity;
import com.mortals.xhx.module.sst.service.SstSkinService;

import java.util.ArrayList;
import java.util.List;

/**
* SstSkinService
* 皮肤管理 service实现
*
* @author zxfei
* @date 2022-12-21
*/
@Service("sstSkinService")
public class SstSkinServiceImpl extends AbstractCRUDServiceImpl<SstSkinDao, SstSkinEntity, Long> implements SstSkinService {

    /** 自助终端设备产品名称 */
    @Value("${system.productName:自助服务终端}")
    private String productName;

    @Autowired
    private ISinkBaseFeign iSinkBaseFeign;

    @Autowired
    private IDeviceFeign deviceFeign;

    @Override
    public Result<SstSkinEntity> find(SstSkinEntity entity, PageInfo pageInfo, Context context) throws AppException {
//        SstSkinEntity newParams = this.findBefore(entity, pageInfo, context);
//        Result<SstSkinEntity> result = this.dao.getList(newParams, pageInfo);
//        this.findAfter(entity, pageInfo, context, result.getList());
//        return result;
        if(entity.getSiteId()==null){
            throw new AppException("站点id不能为空");
        }
        DevicePdu devicePdu = new DevicePdu();
        devicePdu.setProductName(productName);
        Rest<RespData<List<DevicePdu>>> restDevice = deviceFeign.list(devicePdu);
        Long sinkIdUse = 0l;
        if(restDevice.getCode().equals(YesNoEnum.YES.getValue())){
            if(CollectionUtils.isNotEmpty(restDevice.getData().getData())){
                sinkIdUse = restDevice.getData().getData().get(0).getId();
            }
        }
        Result<SstSkinEntity> result = new Result();
        SinkBasePdu skinBasePdu = new SinkBasePdu();
        skinBasePdu.setProductName(productName);
        Rest<RespData<List<SinkBasePdu>>> rest = iSinkBaseFeign.list(skinBasePdu);
        if(rest.getCode().equals(YesNoEnum.YES.getValue())){
            List<SstSkinEntity> list = new ArrayList<>();
            if(CollectionUtils.isNotEmpty(rest.getData().getData())){
                for (SinkBasePdu pdu:rest.getData().getData()) {
                    SstSkinEntity skinEntity = new SstSkinEntity();
                    skinEntity.setSiteId(entity.getSiteId());
                    skinEntity.setSkinName(pdu.getName());
                    skinEntity.setPreviewImagePath(pdu.getPreviewImagePath());
                    skinEntity.setSkinId(pdu.getId());
                    if(pdu.getId()==sinkIdUse){
                        skinEntity.setStatus(1);
                    }else {
                        skinEntity.setStatus(0);
                    }
                    list.add(skinEntity);
                }
            }
            result.setList(list);
            result.setDict(rest.getDict());
        }
        return result;
    }

    @Override
    public SstSkinEntity update(SstSkinEntity entity, Context context) throws AppException {
        return entity;
    }

    @Override
    public SstSkinEntity save(SstSkinEntity entity, Context context) throws AppException {
        return entity;
    }

    @Override
    public void settingSkin(SstSkinEntity sstSkinEntity, String authorization) {
        if(sstSkinEntity.getSiteId()==null){
            throw new AppException("站点id不能为空");
        }
        DevicePdu devicePdu = new DevicePdu();
        devicePdu.setProductName(productName);
        Rest<RespData<List<DevicePdu>>> restDevice = deviceFeign.list(devicePdu);
        if(restDevice.getCode().equals(YesNoEnum.YES.getValue())){
            if(CollectionUtils.isNotEmpty(restDevice.getData().getData())){
                for (DevicePdu device:restDevice.getData().getData()){
                    DevicePdu update = new DevicePdu();
                    update.setId(device.getId());
                    update.setSkinId(sstSkinEntity.getSkinId());
                    deviceFeign.save(update,authorization);
                }
            }
        }
    }
}